#!/bin/bash

declare VERSION="0.0.6a-jha"

# History (as always in the source code :-)
#
# 0.0.6a-jha
#	- fixed a bug that prevented the SLAVE mode from working
#
# 0.0.6-jha
#	- major rework
#	- added config option to disable either NTSC or PAL support
#	- added mplayer's '-speed' option; is disabled by default (see
#	  config file), because it didn't work as expected for me
#	- rudimentary sub titles support:
#	  if a sub titles file (.sub suffix) with exactly the
#	  same name as the played video file exists, it is used
#	  (not really useful yet)
#	- changed the was the config file is parsed:
#	  let's do it the simple way, just source the file
# =================================================
# --->  IMPORTANT! you MUST use the new config file
# =================================================
#
# 0.0.5d-jha
#       - added AC3 support (thanks to Christian Jacobsen
#         <christian.jacobsen -at- stageholding.de>)
#       - better support for mplayers SLAVE mode
#       - fixed some scaling problems
#       - some minor clean-ups
#
# 0.0.5c-jha
#       - play videos with 24 or 23.976 fps as NTSC, not PAL
#
# 0.0.5b-jha
#       - Improved getvidxy - thanks to new mplayer option '-identify'
#         it gets now the correct values for X-/Y-resolution
#         and the framerate (if defined in the video file)
#         for all media files
#       - reworked the scaling calculation to scale only if needed
#         and if so, scale as little as possible to save CPU power
#       - distinguish between NTSC and PAL files; play them with
#         the right TV norm
#
# 0.0.4	Improved getvidxy - No Temp Files needed anymore !
#
# 0.0.3	Seems to work fine - still lacks support for X x 432 Files
#	What about MPEG Files ? 
#


function initialize () {
  # source config file
  source "$CFGFIL"

  # Debug Mode ?
  if test -z "$DEBUG"; then DEBUG=false; fi

  # use AC3?
  if test -z "$USEAC3"; then USEAC3=false; fi
  debugvar USEAC3 "$USEAC3"

  # AC3 command line
  if test -z "$AC3AOUT"; then
    errorcfg AC3AOUT
    exit
  else
    debugvar AC3AOUT "$AC3AOUT"
  fi

  # can do PAL?
  if test -z "$PAL"; then PAL=false; fi
  debugvar PAL "$PAL"

  # can do NTSC?
  if test -z "$NTSC"; then NTSC=false; fi
  debugvar NTSC "$NTSC"

  if ! test $NTSC == "true" -o $PAL == "true"; then
    echolog "*** FATAL: Config Options NTSC and PAL both set to false ... Exiting."
    exit
  fi

  # use speed setting?
  if test -z "$USE_SPEED"; then USE_SPEED=false; fi
  debugvar USE_SPEED "$USE_SPEED"

  # read MPLAYER Config
  if test -z "$MPLAYER"; then errorcfg MPLAYER; exit; else debugvar MPLAYER "$MPLAYER"; fi

  if test -z "$VOP"; then errorcfg VOP; exit; else debugvar VOP "$VOP"; fi

  if ! test -z $CACHE; then
    CACHESTR="-cache $CACHE"; debugvar CACHE "$CACHE"; debugvar CACHESTR "$CACHESTR"
  else
    echolog "*** Option CACHE not set in Config File - calling mplayer without Cache !"
  fi

  if test -z "$FRAMEDROP"; then FRAMEDROP=false; fi
  debugvar FRAMEDROP "$FRAMEDROP"
  if $FRAMEDROP; then FDSTR="-framedrop"; fi 
  debugvar FDSTR "$FDSTR"

  declare LIRCSTR="" # no extra Lirc option !
  if ! test -z "$LIRCRC"; then LIRCSTR="-lircconf $LIRCRC"; fi
  debugvar LIRCRC "$LIRCRC"; debugvar LIRCSTR "$LIRCSTR"

  if test "$SLAVE" != "SLAVE"; then
    REMOTE="$LIRCSTR"
  else
    REMOTE="-slave -nolirc"
  fi
  debugvar REMOTE "$REMOTE"

  if ! test -z "$USERDEF"; then echolog "*** Use Option USERDEF at your own risk !"; fi
  debugvar USERDEF "${USERDEF[*]}"

  if test -z $XResPAL; then errorcfg XResPAL; exit; else debugvar XResPAL "${XResPAL[*]}"; fi
  if test -z $XResNTSC; then errorcfg XResNTSC; exit; else debugvar XResNTSC "${XResNTSC[*]}"; fi
  return;
}


function getvidxy () {

  # Aufruf: getvidxy
  # Ermittelt x und y Aufloesung des FILEs ! 
  # Ausgabe: in Variable ORIG_X und ORIG_Y (global)

  # Variablen Definitionen
  local TEMP1
  ORIG_X=0; ORIG_Y=0

  TEMP1=`$MPLAYER -identify "$FILE" 2>&1 | grep -i -e "^ID_"`
  debugmsg "OutputFromMPLAYER:" "$TEMP1"

  ORIG_X=`echo "$TEMP1"|grep ID_VIDEO_WIDTH|cut -d"=" -f2`
  debugmsg "parsed output for ORIG_X:" $ORIG_X

  ORIG_Y=`echo "$TEMP1"|grep ID_VIDEO_HEIGHT|cut -d"=" -f2`
  debugmsg "parsed output for ORIG_Y:" $ORIG_Y

  ORIG_FPS=`echo "$TEMP1"|grep ID_VIDEO_FPS|cut -d"=" -f2`
  debugmsg "parsed output for ORIG_FPS:" $ORIG_FPS
  test $ORIG_FPS == "14.985" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS
  test $ORIG_FPS == "15.000" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS
  test $ORIG_FPS == "30.000" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS

  ORIG_ASPECT=`echo "$TEMP1"|grep ID_VIDEO_ASPECT|cut -d"=" -f2`
  debugmsg "parsed output for ORIG_ASPECT:" $ORIG_ASPECT

  AUDIO_CODEC=`echo "$TEMP1"|grep ID_AUDIO_CODEC|cut -d"=" -f2`
  debugmsg "parsed output for AUDIO_CODEC:" $AUDIO_CODEC

  return;
}


function calcnewxy () {
  local -i MAX_X

  if test $ORIG_X -gt 352 -o $ORIG_Y -gt $HALF_Y; then
    MAX_X=$((4*$FULL_Y/3)) ; debugvar MAX_X $MAX_X
  else
    MAX_X=$((4*HALF_Y/3)) ; debugvar MAX_X $MAX_X
  fi

  NEW_Y=$(($MAX_X*ORIG_Y/$ORIG_X)); debugvar NEW_Y $NEW_Y
  echolog "*** INFO: For Sqare Pixels we would scale to $MAX_X x $NEW_Y ..."
  test $NEW_Y -gt $FULL_Y && NEW_Y=$FULL_Y 	# Force FullScreen / Verzichtet auf Aspect Ratio !

  return;
}


function choosebestx () {
  local -i MAX_X INDEX
  local XResTEMP
  declare repeat=true
  INDEX=0

  if test $FULL_Y -eq 480; then
    XResTEMP=(${XResNTSC[*]})
  else
    XResTEMP=(${XResPAL[*]})
  fi
  debugvar XResTEMP "${XResTEMP[*]}"

  ANZAHL=${#XResTEMP[*]}
  debugvar AnzahlVonXResTEMP $ANZAHL

  until ! $repeat; do
    if test ${XResTEMP[$INDEX]} -ge $ORIG_X; then repeat=false; NEW_X=${XResTEMP[$INDEX]}; fi
    INDEX=$(($INDEX+1))
    if test $INDEX -ge $ANZAHL; then repeat=false; NEW_X=${XResTEMP[$ANZAHL-1]}; fi
  done
  debugvar NEW_X $NEW_X

  return;
}


choosebesty () {
  if test \( $ORIG_Y -eq $HALF_Y -a $ORIG_X -eq 352 \) -o $ORIG_Y -eq $FULL_Y; then
    REAL_Y=$ORIG_Y
    NEW_Y=$ORIG_Y
    NEW_X=$ORIG_X
    debugmsg "setting NEW = ORIG"
  else
    if test $ORIG_X -gt 352 -o $ORIG_Y -gt $HALF_Y; then
      REAL_Y=$FULL_Y
      debugmsg "setting REAL_Y = FULL_Y"
    else
      REAL_Y=$HALF_Y
      debugmsg "setting REAL_Y = HALF_Y"
    fi
  fi
  return;
}


function set_ntsc () {
  HALF_Y=240
  FULL_Y=480
  NEW_FPS="29.970"
  return;
}


function set_pal () {
  HALF_Y=288
  FULL_Y=576
  NEW_FPS="25.000"
  return;
}


# UNUSED
function aspectratio () {
# Direkt aus 2divx uebernommen - noch anpassen :-)

# Aufruf: aspectratio /video0/..../001.vdr
# Ausgabe: in Variable $ASPECT (global)

# Variablen Definitionen
local TEMP1
local -a TEMP2[0]
local -i ANZAHL i

ASPECT=2	# Default Value (4:3) falls nicht bestimmt werden kann !

$ENCEXE "$1" -frames 0 -o temp_weg.avi 2>$WRKTMP/temp_weg.txt >$WRKTMP/temp_weg2.txt
TEMP1=`cat $WRKTMP/temp_weg.txt $WRKTMP/temp_weg2.txt | grep -i -e "aspect"`
# Old: # TEMP1=`$ENCEXE $1 -frames 0 -o temp_weg.avi | grep -i -e "aspect"`
TEMP2=($TEMP1)
ANZAHL=${#TEMP2[*]}
i=-1

while i=i+1 ; test $i -le $((ANZAHL-1)); do
# Achtung geht davon aus, dass die Ausgabe von Mencoder lautet .. ( aspect x ) ..
    if test ${TEMP2[$i]#(} == "aspect" ; then ASPECT=${TEMP2[$((i+1))]%)}; fi
done
# ggf erzeugtes temp_weg.avi wieder entfernen !!!
if test -f $WRKTMP/temp_weg.txt; then rm $WRKTMP/temp_weg.txt; fi
if test -f $WRKTMP/temp_weg2.txt; then rm $WRKTMP/temp_weg2.txt; fi
if test -f $WRKTMP/temp_weg.avi; then rm $WRKTMP/temp_weg.avi; fi

return;
}


function checkforac3 () {
  if test "X$AUDIO_CODEC" == "Xa52" && $USEAC3; then AOUT="$AC3AOUT"; fi
  return; 
}


function checktvnorm () {
  case $ORIG_FPS in
    "23.976"|"24.000")
      debugmsg "Film"
      if $NTSC; then
        set_ntsc
      else
        set_pal
        SPEED="-speed 25:24"
      fi
    ;;

    "29.970")
      debugmsg "NTSC"
      if $NTSC; then
        set_ntsc
      else
        set_pal
        SPEED="-speed 25:30"
      fi
    ;;

    *)
      debugmsg "PAL or unknown"
      if $PAL; then
        set_pal
      else
        set_ntsc
        SPEED="-speed 30:25"
      fi
    ;;
  esac
  return;
}


function echolog () {
  # prints the String on stdout and into /var/log/messages using logger !
  echo "$1"
  logger "$1"
  return;
}


function errorcfg () {
  echolog "*** FATAL: Config Option $1 not found in Config File ... Exiting."
  exit;
}


function debugvar () {
  if $DEBUG; then echolog "*** DEBUG: Variable $1 has value \"$2\""; fi
  return;
}


function debugmsg () {
  if $DEBUG; then echolog "*** DEBUG: $1 \"$2\""; fi
  return;
}


# Begin Main !
#
#
# ---------------------------------------------------------------------------
#
#

declare -i ORIG_X ORIG_Y
declare -i NEW_X NEW_Y
declare -i HALF_Y FULL_Y
declare ORIG_FPS NEW_FPS ORIG_ASPECT AUDIO_CODEC
declare CMDLINE AOUT REMOTE
declare FILE="$1"
declare SLAVE="$2"
declare AOUT="-ao mpegpes -afm 9"
declare -a USERDEF[0]
declare -a XResPAL[0]
declare -a XResNTSC[0]
declare FDSTR=""

echolog "*** Starting mplayer.sh Version $VERSION"

if test -z "$FILE"; then echolog "*** USAGE: mplayer.sh <File_to_be_played>"; exit; fi
if ! test -f "$FILE"; then echolog "*** ERROR: Make sure $FILE exists - otherwise it cannot be played ;-)"; exit; fi

# Check if Config File exists !
declare CFGFIL="${0}.conf"
debugvar CFGFIL $CFGFIL
if ! test -f $CFGFIL; then echolog "*** FATAL: mplayer.sh.conf not found !!! Exiting." ; exit; fi

# read config file and initialize the variables
initialize

# Try to determine the video attributes
if $DEBUG; then echolog "*** DEBUG: Calling getvidxy function to analyze source video stream ..."; fi
getvidxy
echolog "*** INFO: Source Video has Resolution of $ORIG_X x $ORIG_Y ..."
if test $ORIG_X -eq 0 -o $ORIG_Y -eq 0; then echolog "*** FATAL: Illegal Resolution ..."; exit; fi

NEW_FPS=$ORIG_FPS

checkforac3
checktvnorm
calcnewxy
choosebestx
choosebesty
if test $NEW_X -eq 0 -o $NEW_Y -eq 0; then echolog "*** FATAL: Illegal Resolution ..."; exit; fi

if ! $USE_SPEED; then SPEED=""; fi

CMDLINE="$MPLAYER -vo mpegpes $AOUT -vop $VOP:$NEW_FPS,expand=$NEW_X:$REAL_Y:-1:-1:1,scale=$NEW_X:$NEW_Y $SPEED $FDSTR $CACHESTR $REMOTE $USERDEF"
debugvar CMDLINE "$CMDLINE"
exec $CMDLINE "$FILE"
exit
